@@ -1,7 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2021 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
@@ -36,7 +36,7 @@ def tamper(payload, **kwargs):
 
     >>> random.seed(0)
     >>> tamper('1 AND 9227=9227')
-    '1%23nVNaVoPYeva%0AAND%23ngNvzqu%0A9227=9227'
+    '1%23upgPydUzKpMX%0AAND%23RcDKhIr%0A9227=9227'
     """
 
     retVal = ""
